#!/bin/bash
#
#

ScriptHome=$(echo $HOME)
MY_PATH="`dirname \"$0\"`"
DEST_PATH="/Users/$USER/Library/Application Support/Legacy of Empires"
cd "$MY_PATH"

ARCH=$(uname -m)

if [ "$ARCH" = "arm64" ]; then
    innoextract_bin="../bin/innoextract_arm/./innoextract"
else
    innoextract_bin="../bin/innoextract_x86/./innoextract"
fi

CAESAR_GOG_ID="1207658835"
EMPEROR_GOG_ID="1598055675"
PHARAOH_GOG_ID="1207659046"
ZEUS_GOG_ID="1207659039"

LANGUAGE=$( defaults read -g AppleLanguages | head -n2 | tail -n 1 | tr -d '", ' )

if [ ! -d "$DEST_PATH"/ws_patches ]; then
    mkdir -p "$DEST_PATH/ws_patches/emperor" "$DEST_PATH/ws_patches/pharaoh" "$DEST_PATH/ws_patches/zeus" "$DEST_PATH"/ger_patches
fi

if [ ! -d "/Users/$USER/Documents/Legacy of Empires" ]; then
    mkdir -p "/Users/$USER/Documents/Legacy of Empires/Caesar_Saves" "/Users/$USER/Documents/Legacy of Empires/Emperor_Saves" "/Users/$USER/Documents/Legacy of Empires/Pharaoh_Saves" "/Users/$USER/Documents/Legacy of Empires/Zeus_Saves"
fi

function init()
{
    echo ""
}

function _helpDefaultRead()
{
    VAL=$1
    
    if [ ! -z "$VAL" ]; then
        defaults read "${ScriptHome}/Library/Preferences/slsoft.legacy-of-empires.plist" "$VAL"
    fi
}

function _helpDefaultWrite()
{
    VAL=$1
    local VAL1=$2
    
    if [ ! -z "$VAL" ] || [ ! -z "$VAL1" ]; then
        defaults write "${ScriptHome}/Library/Preferences/slsoft.legacy-of-empires.plist" "$VAL" "$VAL1"
    fi
}

function install_wrapper()
{
    check_online=$( curl -sS https://www.sl-soft.de/extern/software/legacy_of_empires/wrapper )
    check_local=$( cat "$DEST_PATH/Legacy of Empires.app/.version" )
    
    if [ "$check_local" = "" ]; then
        check_local="999"
    fi
    
    if [ "$check_online" = "" ]; then
        check_online="0"
    fi
    
    if [ "$check_online" -gt "$check_local" ]; then
        if [[ "$LANGUAGE" == "de-DE" ]]; then
            _helpDefaultWrite "StatusText" "Herunterladen/Installieren vom Wine Wrapper Update"
        else
            _helpDefaultWrite "StatusText" "Downloading/Installing Wine Wrapper Update"
        fi

        if [ -d "$DEST_PATH/Legacy of Empires.app" ]; then
            rm -rf "$DEST_PATH/Legacy of Empires.app"
        fi

        curl -sS https://www.sl-soft.de/extern/software/legacy_of_empires/wrapper.7z -o "$DEST_PATH/wrapper.7z"
        ../bin/./7z e "$DEST_PATH/wrapper.7z" -o/private/tmp > /dev/null 2>&1
        hdiutil attach /private/tmp/Legacy\ of\ Empires.dmg -nobrowse -quiet -noverify
        if cp -RH /Volumes/Legacy\ of\ Empires/Legacy\ of\ Empires.app "$DEST_PATH"; then
            diskutil eject /Volumes/Legacy\ of\ Empires
            rm -f "/private/tmp/Legacy of Empires.dmg"
            _done
        else
            error_occured
        fi
    else
        if [ ! -d "$DEST_PATH/Legacy of Empires.app" ]; then
            if [[ "$LANGUAGE" == "de-DE" ]]; then
                _helpDefaultWrite "StatusText" "Herunterladen/Installieren vom Wine Wrapper"
            else
                _helpDefaultWrite "StatusText" "Downloading/Installing Wine Wrapper"
            fi
            if [ ! -f "$DEST_PATH/wrapper.7z" ]; then
                curl -sS https://www.sl-soft.de/extern/software/legacy_of_empires/wrapper.7z -o "$DEST_PATH/wrapper.7z"
            fi
            ../bin/./7z e "$DEST_PATH/wrapper.7z" -o/private/tmp > /dev/null 2>&1
            hdiutil attach /private/tmp/Legacy\ of\ Empires.dmg -nobrowse -quiet -noverify
            if cp -RH /Volumes/Legacy\ of\ Empires/Legacy\ of\ Empires.app "$DEST_PATH"; then
                diskutil eject /Volumes/Legacy\ of\ Empires
                rm -f "/private/tmp/Legacy of Empires.dmg"
                _done
            else
                error_occured
            fi
        fi
    fi
    
    _helpDefaultWrite "StatusText" ""
}

function install_caesar()
{
    check_online=$( curl -sS https://www.sl-soft.de/extern/software/legacy_of_empires/augustus )
    check_local=$( cat "$DEST_PATH"/caesar/augustus.app/.version )
    
    if [ "$check_local" = "" ]; then
        check_local="999"
    fi
    
    if [ "$check_online" = "" ]; then
        check_online="0"
    fi
    
    if [ ! -f "$DEST_PATH"/caesar/augustus.app/Contents/Resources/c3.exe ]; then
        _helpDefaultWrite "Busy" "1"
FILE=$(osascript <<EOT
    set chosenFile to choose file with prompt "Select GOG Setup EXE \"setup_caesartm_........\""
    POSIX path of chosenFile
EOT
)
        GOG_ID=$( "$innoextract_bin" -i "$FILE" | grep ID | sed 's/.*is\ //g' )
        if [ "$GOG_ID" = "$CAESAR_GOG_ID" ]; then
            if [[ "$LANGUAGE" == "de-DE" ]]; then
                _helpDefaultWrite "StatusText" "Ceasar III wird installiert"
            else
                _helpDefaultWrite "StatusText" "Installing Caesar III"
            fi
            curl -sS https://www.sl-soft.de/extern/software/legacy_of_empires/caesar/caesar_app.7z -o /private/tmp/caesar_app.7z
            ../bin/./7z x /private/tmp/caesar_app.7z -o"$DEST_PATH"/caesar -aoa > /dev/null 2>&1
            rm /private/tmp/caesar_app.7z
            if "$innoextract_bin" -q -e -s "$FILE" -d "$DEST_PATH"/caesar/augustus.app/Contents/Resources; then
                echo "OK"
            else
                error_occured
            fi
            mkdir -p "/Users/$USER/Library/Application Support/augustus/augustus"
            echo -n "./" > "/Users/$USER/Library/Application Support/augustus/augustus"/user_dir.txt
            mkdir "/Users/$USER/Library/Application Support/Legacy of Empires/caesar/augustus.app/Contents/Resources/config"
            curl -sS https://www.sl-soft.de/extern/software/legacy_of_empires/caesar/augustus.ini -o "/Users/$USER/Library/Application Support/Legacy of Empires/caesar/augustus.app/Contents/Resources/config"/augustus.ini
            ln -s "/Users/$USER/Documents/Legacy of Empires/Caesar_Saves" "/Users/$USER/Library/Application Support/Legacy of Empires/caesar/augustus.app/Contents/Resources/savegames"
            _done
            rm -rf "$DEST_PATH/caesar/augustus.app/Contents/Resources/__redist" "$DEST_PATH/caesar/augustus.app/Contents/Resources/__support" "$DEST_PATH/caesar/augustus.app/Contents/Resources/commonappdata" "$DEST_PATH/caesar/augustus.app/Contents/Resources/tmp" "$DEST_PATH/caesar/augustus.app/Contents/Resources/app"
        else
            error_gog
        fi
    else
        if [ "$check_online" -gt "$check_local" ]; then
            if [[ "$LANGUAGE" == "de-DE" ]]; then
                _helpDefaultWrite "StatusText" "Installiere Caesar III Update"
            else
                _helpDefaultWrite "StatusText" "Installing Caesar III Update"
            fi
            if curl -sS https://www.sl-soft.de/extern/software/legacy_of_empires/caesar/caesar_app.7z -o /private/tmp/caesar_app.7z; then
                rm -rf "$DEST_PATH"/caesar/augustus.app/Contents/Resources/assets
                rm -rf "$DEST_PATH"/caesar/augustus.app/Contents/Frameworks
                rm -rf "$DEST_PATH"/caesar/augustus.app/Contents/MacOS
                rm -rf "$DEST_PATH"/caesar/augustus.app/Contents/_CodeSignature
                ../bin/./7z x /private/tmp/caesar_app.7z -o"$DEST_PATH"/caesar -aoa > /dev/null 2>&1
                rm /private/tmp/caesar_app.7z
            else
                error_occured
            fi
        fi
    fi
    
    _helpDefaultWrite "Busy" "0"
}

function install_emperor()
{
    if [ ! -f "$DEST_PATH"/emperor/Emperor.exe ]; then
FILE=$(osascript <<EOT
    set chosenFile to choose file with prompt "Select GOG Setup EXE \"setup_emperor_rise_of_the_middle_kingdom_........\""
    POSIX path of chosenFile
EOT
)
        GOG_ID=$( "$innoextract_bin" -i "$FILE" | grep ID | sed 's/.*is\ //g' )
        if [ "$GOG_ID" = "$EMPEROR_GOG_ID" ]; then
            if [[ "$LANGUAGE" == "de-DE" ]]; then
                _helpDefaultWrite "StatusText" "Der erste Kaiser: Aufstieg des Reichs der Mitte wird installiert"
            else
                _helpDefaultWrite "StatusText" "Installing Emperor"
            fi
            if "$innoextract_bin" -q -e -s "$FILE" -d "$DEST_PATH"/emperor; then
                 echo "OK"
            else
                error_occured
            fi
            mv "$DEST_PATH"/emperor/app/* "$DEST_PATH"/emperor
            curl -sS https://www.sl-soft.de/extern/software/legacy_of_empires/emperor/EmperorMap.inf -o "$DEST_PATH"/emperor/EmperorMap.inf
            curl -sS https://www.sl-soft.de/extern/software/legacy_of_empires/emperor/Emperor.inf -o "$DEST_PATH"/emperor/Emperor.inf
            install_wrapper
            
            rm -r "/Users/$USER/Library/Application Support/Legacy of Empires/emperor/Save"
            ln -s "/Users/$USER/Documents/Legacy of Empires/Emperor_Saves" "/Users/$USER/Library/Application Support/Legacy of Empires/emperor/Save"
            
            rm -rf "$DEST_PATH"/emperor/app "$DEST_PATH"/emperor/__support "$DEST_PATH"/emperor/Binks "$DEST_PATH"/emperor/tmp
           
            _done
        else
            error_gog
        fi
    fi
    
    _helpDefaultWrite "StatusText" ""
}

function install_pharaoh()
{
    if [ ! -f "$DEST_PATH"/pharaoh/Pharaoh.exe ]; then
FILE=$(osascript <<EOT
    set chosenFile to choose file with prompt "Select GOG Setup EXE \"setup_pharaoh_gold_........\""
    POSIX path of chosenFile
EOT
)
        GOG_ID=$( "$innoextract_bin" -i "$FILE" | grep ID | sed 's/.*is\ //g' )
        if [ "$GOG_ID" = "$PHARAOH_GOG_ID" ]; then
        
            if [[ "$LANGUAGE" == "de-DE" ]]; then
                _helpDefaultWrite "StatusText" "Pharaoh + Cleopatra wird installiert"
            else
                _helpDefaultWrite "StatusText" "Installing Pharaoh + Cleopatra"
            fi
            
            if "$innoextract_bin" -q -e -s "$FILE" -d "$DEST_PATH"/pharaoh; then
                echo "OK"
            else
                error_occured
            fi
            mv "$DEST_PATH"/pharaoh/app/* "$DEST_PATH"/pharaoh
            mkdir "$DEST_PATH"/pharaoh/Save
            curl -sS https://www.sl-soft.de/extern/software/legacy_of_empires/pharaoh/Pharaohmap.inf -o "$DEST_PATH"/pharaoh/Pharaohmap.inf
            curl -sS https://www.sl-soft.de/extern/software/legacy_of_empires/pharaoh/Pharaoh.inf -o "$DEST_PATH"/pharaoh/Pharaoh.inf
            install_wrapper
            
            rm -r "/Users/$USER/Library/Application Support/Legacy of Empires/pharaoh/Save"
            ln -s "/Users/$USER/Documents/Legacy of Empires/Pharaoh_Saves" "/Users/$USER/Library/Application Support/Legacy of Empires/pharaoh/Save"
            
            rm -rf "$DEST_PATH"/pharaoh/app "$DEST_PATH"/pharaoh/__support "$DEST_PATH"/pharaoh/tmp "$DEST_PATH"/pharaoh/Binks/High/Intro_big.bik

            _done
        else
            error_gog
        fi
    fi
    
    _helpDefaultWrite "StatusText" ""
}

function install_zeus()
{
    if [ ! -f "$DEST_PATH"/zeus/Zeus.exe ]; then
FILE=$(osascript <<EOT
    set chosenFile to choose file with prompt "Select GOG Setup EXE \"setup_zeus_and_poseidon_*\""
    POSIX path of chosenFile
EOT
)
        GOG_ID=$( "$innoextract_bin" -i "$FILE" | grep ID | sed 's/.*is\ //g' )
        if [ "$GOG_ID" = "$ZEUS_GOG_ID" ]; then
            if [[ "$LANGUAGE" == "de-DE" ]]; then
                _helpDefaultWrite "StatusText" "Zeus + Poseidon wird installiert"
            else
                _helpDefaultWrite "StatusText" "Installing Zeus + Poseidon"
            fi
            if "$innoextract_bin" -q -e -s "$FILE" -d "$DEST_PATH"/zeus; then
                echo "OK"
            else
                error_occured
            fi
            mv "$DEST_PATH"/zeus/app/* "$DEST_PATH"/zeus
            rm -rf "$DEST_PATH"/zeus/app "$DEST_PATH"/zeus/commonappdata
            curl -sS https://www.sl-soft.de/extern/software/legacy_of_empires/zeus/ZeusMap.inf -o "$DEST_PATH"/zeus/ZeusMap.inf
            curl -sS https://www.sl-soft.de/extern/software/legacy_of_empires/zeus/Zeus.inf -o "$DEST_PATH"/zeus/Zeus.inf
            install_wrapper
            
            rm -r "/Users/$USER/Library/Application Support/Legacy of Empires/zeus/Save"
            ln -s "/Users/$USER/Documents/Legacy of Empires/Zeus_Saves" "/Users/$USER/Library/Application Support/Legacy of Empires/zeus/Save"
            
            rm -rf "$DEST_PATH"/zeus/app "$DEST_PATH"/zeus/__support "$DEST_PATH"/zeus/tmp "$DEST_PATH"/zeus/Binks

            _done
        else
            error_gog
        fi
    fi
    
    _helpDefaultWrite "StatusText" ""
}

function german_caesar()
{
    if [ ! -f "$DEST_PATH"/caesar/.german ]; then
        download_german
        if [ ! -f "$DEST_PATH/ger_patches/caesar_deutsch.7z" ]; then
            curl -sS https://www.sl-soft.de/extern/software/legacy_of_empires/caesar/caesar_deutsch.7z -o "$DEST_PATH/ger_patches/caesar_deutsch.7z"
        fi
        if ../bin/./7z x "$DEST_PATH/ger_patches/caesar_deutsch.7z" -o"$DEST_PATH"/caesar/augustus.app/Contents/Resources -aoa > /dev/null 2>&1; then
            check_sum=$( shasum "$DEST_PATH"/caesar/augustus.app/Contents/Resources/C3_mm.eng | awk '{print $1}' )
            if [ "$check_sum" = "cfe91c7b71465b2e095ff4ed6a1e6b6420b9fdc2" ]; then
                mv "$DEST_PATH"/caesar/augustus.app/Contents/Resources/C3_mm.eng "$DEST_PATH"/caesar/augustus.app/Contents/Resources/C3_mm.eng_Ori
                ../bin/xdelta/./xdelta3 -d -s "$DEST_PATH"/caesar/augustus.app/Contents/Resources/C3_mm.eng_Ori ../patches/caesar/C3_mm.xdelta "$DEST_PATH"/caesar/augustus.app/Contents/Resources/C3_mm.eng
            fi
            check_sum=$( shasum "$DEST_PATH"/caesar/augustus.app/Contents/Resources/c3.eng | awk '{print $1}' )
            if [ "$check_sum" = "aefd959ece6e0c278abdc9264c541fc07052eaff" ]; then
                mv "$DEST_PATH"/caesar/augustus.app/Contents/Resources/c3.eng "$DEST_PATH"/caesar/augustus.app/Contents/Resources/c3.eng_Ori
                ../bin/xdelta/./xdelta3 -d -s "$DEST_PATH"/caesar/augustus.app/Contents/Resources/c3.eng_Ori ../patches/caesar/c3.xdelta "$DEST_PATH"/caesar/augustus.app/Contents/Resources/c3.eng
            fi
            echo "1" > "$DEST_PATH"/caesar/.german
            _done
        else
            error_occured
        fi
    fi
    
    _helpDefaultWrite "StatusText" ""
}

function german_emperor()
{
    if [ ! -f "$DEST_PATH"/emperor/.german ]; then
        download_german
        if [ ! -f "$DEST_PATH/ger_patches/emperor_deutsch.7z" ]; then
            curl -sS https://www.sl-soft.de/extern/software/legacy_of_empires/emperor/emperor_deutsch.7z -o "$DEST_PATH/ger_patches/emperor_deutsch.7z"
        fi
        if ../bin/./7z x "$DEST_PATH/ger_patches/emperor_deutsch.7z" -o"$DEST_PATH"/emperor -aoa > /dev/null 2>&1; then
            check_sum=$( shasum "$DEST_PATH"/emperor/EmperorMM.eng | awk '{print $1}' )
            if [ "$check_sum" = "e05867c3f61101f3fc1161f452ee8a33f1b5dca9" ]; then
                mv "$DEST_PATH"/emperor/EmperorMM.eng "$DEST_PATH"/emperor/EmperorMM.eng_Ori
                ../bin/xdelta/./xdelta3 -d -s "$DEST_PATH"/emperor/EmperorMM.eng_Ori ../patches/emperor/EmperorMM.xdelta "$DEST_PATH"/emperor/EmperorMM.eng
            fi
            check_sum=$( shasum "$DEST_PATH"/emperor/EmperorText.eng | awk '{print $1}' )
            if [ "$check_sum" = "8b559cd60ab7da4a044fc2263d68394876939c9a" ]; then
                mv "$DEST_PATH"/emperor/EmperorText.eng "$DEST_PATH"/emperor/EmperorText.eng_Ori
                ../bin/xdelta/./xdelta3 -d -s "$DEST_PATH"/emperor/EmperorText.eng_Ori ../patches/emperor/EmperorText.xdelta "$DEST_PATH"/emperor/EmperorText.eng
            fi
        
            echo "1" > "$DEST_PATH"/emperor/.german
            _done
        else
            error_occured
        fi
    fi

    #exe e5eb9d3f8a02236b65823f9ab7ec3edf75a805f5
    
    _helpDefaultWrite "StatusText" ""
}

function german_pharaoh()
{
    if [ ! -f "$DEST_PATH"/pharaoh/.german ]; then
        download_german
        
        if [ ! -f "$DEST_PATH/ger_patches/pharaoh_deutsch.7z" ]; then
            curl -sS https://www.sl-soft.de/extern/software/legacy_of_empires/pharaoh/pharaoh_deutsch.7z -o "$DEST_PATH/ger_patches/pharaoh_deutsch.7z"
        fi
        if ../bin/./7z x "$DEST_PATH/ger_patches/pharaoh_deutsch.7z" -o"$DEST_PATH"/pharaoh -aoa > /dev/null 2>&1; then
            check_sum=$( shasum "$DEST_PATH"/pharaoh/Pharaoh_MM.eng | awk '{print $1}' )
            if [ "$check_sum" = "274b75342d48191d8f8fca36c38e043475826a7e" ]; then
                mv "$DEST_PATH"/pharaoh/Pharaoh_MM.eng "$DEST_PATH"/pharaoh/Pharaoh_MM.eng_Ori
                ../bin/xdelta/./xdelta3 -d -s "$DEST_PATH"/pharaoh/Pharaoh_MM.eng_Ori ../patches/pharaoh/Pharaoh_MM.xdelta "$DEST_PATH"/pharaoh/Pharaoh_MM.eng
            fi
            check_sum=$( shasum "$DEST_PATH"/pharaoh/Pharaoh_Text.eng | awk '{print $1}' )
            if [ "$check_sum" = "0dd284c9d93276ee728dd8b12cd519e087468f35" ]; then
                mv "$DEST_PATH"/pharaoh/Pharaoh_Text.eng "$DEST_PATH"/pharaoh/Pharaoh_Text.eng_Ori
                ../bin/xdelta/./xdelta3 -d -s "$DEST_PATH"/pharaoh/Pharaoh_Text.eng_Ori ../patches/pharaoh/Pharaoh_Text.xdelta "$DEST_PATH"/pharaoh/Pharaoh_Text.eng
            fi
            echo "1" > "$DEST_PATH"/pharaoh/.german
            _done
        fi
    fi
    
    _helpDefaultWrite "StatusText" ""
}

function german_zeus()
{
    if [ ! -f "$DEST_PATH"/zeus/.german ]; then
        download_german
        if [ ! -f "$DEST_PATH/ger_patches/zeus_deutsch.7z" ]; then
            curl -sS https://www.sl-soft.de/extern/software/legacy_of_empires/zeus/zeus_deutsch.7z -o "$DEST_PATH/ger_patches/zeus_deutsch.7z"
        fi
        if ../bin/./7z x "$DEST_PATH/ger_patches/zeus_deutsch.7z" -o"$DEST_PATH"/zeus -aoa > /dev/null 2>&1; then
            check_sum=$( shasum "$DEST_PATH"/zeus/Zeus_MM.eng | awk '{print $1}' )
            if [ "$check_sum" = "dc910c6fd09f0dcbc710cd438629d779f3bd4b1b" ]; then
                mv "$DEST_PATH"/zeus/Zeus_MM.eng "$DEST_PATH"/zeus/Zeus_MM.eng_Ori
                ../bin/xdelta/./xdelta3 -d -s "$DEST_PATH"/zeus/Zeus_MM.eng_Ori ../patches/zeus/Zeus_MM.xdelta "$DEST_PATH"/zeus/Zeus_MM.eng
            fi
            check_sum=$( shasum "$DEST_PATH"/zeus/Zeus_Text.eng | awk '{print $1}' )
            if [ "$check_sum" = "208ad98d58d5d6179118384e8280f1f44f19ae0c" ]; then
                mv "$DEST_PATH"/zeus/Zeus_Text.eng "$DEST_PATH"/zeus/Zeus_Text.eng_Ori
                ../bin/xdelta/./xdelta3 -d -s "$DEST_PATH"/zeus/Zeus_Text.eng_Ori ../patches/zeus/Zeus_Text.xdelta "$DEST_PATH"/zeus/Zeus_Text.eng
            fi
            echo "1" > "$DEST_PATH"/zeus/.german
            _done
        else
            error_occured
        fi
    fi
    
    _helpDefaultWrite "StatusText" ""
}

function resolution_emperor()
{
        if [ ! -f "$DEST_PATH"/emperor/.resolution ]; then
        install_widescreen
        RESOLUTION=$( _helpDefaultRead "EmperorResolution" )
        if [ ! -f "$DEST_PATH/ws_patches/emperor/$RESOLUTION".7z ]; then
            curl -sS https://www.sl-soft.de/extern/software/legacy_of_empires/emperor/"$RESOLUTION".7z -o "$DEST_PATH/ws_patches/emperor/$RESOLUTION".7z
        fi
        ../bin/./7z x "$DEST_PATH/ws_patches/emperor/$RESOLUTION".7z -o"$DEST_PATH"/emperor -aoa > /dev/null 2>&1
        cp -rf "$DEST_PATH"/emperor/"$RESOLUTION"/. "$DEST_PATH"/emperor
        rm -rf "$DEST_PATH"/emperor/"$RESOLUTION"
        
        if [ ! -f "$DEST_PATH"/emperor/Emperor.exe_Ori ]; then
            cp "$DEST_PATH"/emperor/Emperor.exe "$DEST_PATH"/emperor/Emperor.exe_Ori
        fi
        check_sum=$( shasum "$DEST_PATH"/emperor/Emperor.exe_Ori | awk '{print $1}' )
        if [ "$check_sum" = "e5eb9d3f8a02236b65823f9ab7ec3edf75a805f5" ]; then
            ../bin/xdelta/./xdelta3 -f -d -s "$DEST_PATH"/emperor/Emperor.exe_Ori ../patches/emperor/"$RESOLUTION".xdelta "$DEST_PATH"/emperor/Emperor.exe
        fi
        
        echo "$RESOLUTION" > "$DEST_PATH"/emperor/.resolution
    else
        RESOLUTION_EXISTS=$( cat "$DEST_PATH"/emperor/.resolution )
        RESOLUTION=$( _helpDefaultRead "EmperorResolution" )
        if [ "$RESOLUTION_EXISTS" != "$RESOLUTION" ]; then
            install_widescreen
            if [ ! -f "$DEST_PATH/ws_patches/emperor/$RESOLUTION".7z ]; then
                curl -sS https://www.sl-soft.de/extern/software/legacy_of_empires/emperor/"$RESOLUTION".7z -o "$DEST_PATH/ws_patches/emperor/$RESOLUTION".7z
            fi
            ../bin/./7z x "$DEST_PATH/ws_patches/emperor/$RESOLUTION".7z -o"$DEST_PATH"/emperor -aoa > /dev/null 2>&1
            cp -rf "$DEST_PATH"/emperor/"$RESOLUTION"/* "$DEST_PATH"/emperor
            rm -rf "$DEST_PATH"/emperor/"$RESOLUTION"
            
            if [ ! -f "$DEST_PATH"/emperor/Emperor.exe_Ori ]; then
                cp "$DEST_PATH"/emperor/Emperor.exe "$DEST_PATH"/emperor/Emperor.exe_Ori
            fi
            check_sum=$( shasum "$DEST_PATH"/emperor/Emperor.exe_Ori | awk '{print $1}' )
            if [ "$check_sum" = "e5eb9d3f8a02236b65823f9ab7ec3edf75a805f5" ]; then
                ../bin/xdelta/./xdelta3 -f -d -s "$DEST_PATH"/emperor/Emperor.exe_Ori ../patches/emperor/"$RESOLUTION".xdelta "$DEST_PATH"/emperor/Emperor.exe
            fi
            
            echo "$RESOLUTION" > "$DEST_PATH"/emperor/.resolution
        fi
    fi
    
    _helpDefaultWrite "StatusText" ""
}

function resolution_pharaoh()
{
    if [ ! -f "$DEST_PATH"/pharaoh/.resolution ]; then
        install_widescreen
        RESOLUTION=$( _helpDefaultRead "PharaohResolution" )
        #if [ ! -f "$DEST_PATH/ws_patches/pharaoh/$RESOLUTION".zip ]; then
        #    curl -sS https://www.sl-soft.de/extern/software/legacy_of_empires/pharaoh/"$RESOLUTION".zip -o "$DEST_PATH/ws_patches/pharaoh/$RESOLUTION".zip
        #fi
        #../bin/./7z x "$DEST_PATH/ws_patches/pharaoh/$RESOLUTION".zip -o"$DEST_PATH"/pharaoh/ -aoa > /dev/null 2>&1
        if [ ! -f "$DEST_PATH"/pharaoh/Pharaoh.exe_Ori ]; then
            cp "$DEST_PATH"/pharaoh/Pharaoh.exe "$DEST_PATH"/pharaoh/Pharaoh.exe_Ori
        fi
        check_sum=$( shasum "$DEST_PATH"/pharaoh/Pharaoh.exe_Ori | awk '{print $1}' )
        if [ "$check_sum" = "1262de5ed35eb7746dc577101e87f6763892098f" ]; then
            ../bin/xdelta/./xdelta3 -f -d -s "$DEST_PATH"/pharaoh/Pharaoh.exe_Ori ../patches/pharaoh/"$RESOLUTION".xdelta "$DEST_PATH"/pharaoh/Pharaoh.exe
        fi
        echo "$RESOLUTION" > "$DEST_PATH"/pharaoh/.resolution
    else
        RESOLUTION_EXISTS=$( cat "$DEST_PATH"/pharaoh/.resolution )
        RESOLUTION=$( _helpDefaultRead "PharaohResolution" )
        if [ "$RESOLUTION_EXISTS" != "$RESOLUTION" ]; then
            install_widescreen
            if [ ! -f "$DEST_PATH"/pharaoh/Pharaoh.exe_Ori ]; then
                cp "$DEST_PATH"/pharaoh/Pharaoh.exe "$DEST_PATH"/pharaoh/Pharaoh.exe_Ori
            fi
            check_sum=$( shasum "$DEST_PATH"/pharaoh/Pharaoh.exe_Ori | awk '{print $1}' )
            if [ "$check_sum" = "1262de5ed35eb7746dc577101e87f6763892098f" ]; then
                ../bin/xdelta/./xdelta3 -f -d -s "$DEST_PATH"/pharaoh/Pharaoh.exe_Ori ../patches/pharaoh/"$RESOLUTION".xdelta "$DEST_PATH"/pharaoh/Pharaoh.exe
            fi
            echo "$RESOLUTION" > "$DEST_PATH"/pharaoh/.resolution
        fi
    fi
    
    
    
    
    _helpDefaultWrite "StatusText" ""
}

function resolution_zeus()
{
    if [ ! -f "$DEST_PATH"/zeus/.resolution ]; then
        install_widescreen
        RESOLUTION=$( _helpDefaultRead "ZeusResolution" )
        if [ ! -f "$DEST_PATH/ws_patches/zeus/$RESOLUTION".7z ]; then
            curl -sS https://www.sl-soft.de/extern/software/legacy_of_empires/zeus/"$RESOLUTION".7z -o "$DEST_PATH/ws_patches/zeus/$RESOLUTION".7z
        fi
        ../bin/./7z x "$DEST_PATH/ws_patches/zeus/$RESOLUTION".7z -o"$DEST_PATH"/zeus -aoa > /dev/null 2>&1
        cp -rf "$DEST_PATH"/zeus/"$RESOLUTION"/. "$DEST_PATH"/zeus
        rm -rf "$DEST_PATH"/zeus/"$RESOLUTION"
        
        if [ ! -f "$DEST_PATH"/zeus/Zeus.exe_Ori ]; then
            cp "$DEST_PATH"/zeus/Zeus.exe "$DEST_PATH"/zeus/Zeus.exe_Ori
        fi
        check_sum=$( shasum "$DEST_PATH"/zeus/Zeus.exe_Ori | awk '{print $1}' )
        if [ "$check_sum" = "8dbc88265072ef2e942c881e5f399c5170c772e0" ]; then
            ../bin/xdelta/./xdelta3 -f -d -s "$DEST_PATH"/zeus/Zeus.exe_Ori ../patches/zeus/"$RESOLUTION".xdelta "$DEST_PATH"/zeus/Zeus.exe
        fi
        
        echo "$RESOLUTION" > "$DEST_PATH"/zeus/.resolution
    else
        RESOLUTION_EXISTS=$( cat "$DEST_PATH"/zeus/.resolution )
        RESOLUTION=$( _helpDefaultRead "ZeusResolution" )
        if [ "$RESOLUTION_EXISTS" != "$RESOLUTION" ]; then
            install_widescreen
            if [ ! -f "$DEST_PATH/ws_patches/zeus/$RESOLUTION".7z ]; then
                curl -sS https://www.sl-soft.de/extern/software/legacy_of_empires/zeus/"$RESOLUTION".7z -o "$DEST_PATH/ws_patches/zeus/$RESOLUTION".7z
            fi
            ../bin/./7z x "$DEST_PATH/ws_patches/zeus/$RESOLUTION".7z -o"$DEST_PATH"/zeus -aoa > /dev/null 2>&1
            cp -rf "$DEST_PATH"/zeus/"$RESOLUTION"/* "$DEST_PATH"/zeus
            rm -rf "$DEST_PATH"/zeus/"$RESOLUTION"
            
            if [ ! -f "$DEST_PATH"/zeus/Zeus.exe_Ori ]; then
                cp "$DEST_PATH"/zeus/Zeus.exe "$DEST_PATH"/zeus/Zeus.exe_Ori
            fi
            check_sum=$( shasum "$DEST_PATH"/zeus/Zeus.exe_Ori | awk '{print $1}' )
            if [ "$check_sum" = "8dbc88265072ef2e942c881e5f399c5170c772e0" ]; then
                ../bin/xdelta/./xdelta3 -f -d -s "$DEST_PATH"/zeus/Zeus.exe_Ori ../patches/zeus/"$RESOLUTION".xdelta "$DEST_PATH"/zeus/Zeus.exe
            fi
            
            echo "$RESOLUTION" > "$DEST_PATH"/zeus/.resolution
        fi
    fi
    
    _helpDefaultWrite "StatusText" ""
}

function play_caesar()
{
    install_caesar
    
    if [ ! -L "$DEST_PATH/caesar/augustus.app/Contents/Resources/savegames" ]; then
        rm -rf "$DEST_PATH/caesar/augustus.app/Contents/Resources/savegames"
        ln -s "/Users/$USER/Documents/Legacy of Empires/Caesar_Saves" "$DEST_PATH/caesar/augustus.app/Contents/Resources/savegames"
    fi
    
    if [[ "$LANGUAGE" == "de-DE" ]]; then
        _helpDefaultWrite "StatusText" "Caesar III läuft"
    else
        _helpDefaultWrite "StatusText" "Caesar III is running"
    fi

    _helpDefaultWrite "Busy" "1"
    
    open -W "$DEST_PATH"/caesar/augustus.app
    
    while pgrep -f "augustus.app" > /dev/null; do
        sleep 2
    done
    _helpDefaultWrite "Busy" "0"
    _helpDefaultWrite "StatusText" ""
}

function play_emperor()
{
    install_wrapper
    install_emperor
    resolution_emperor
    
    if [ ! -L "$DEST_PATH/emperor/Save" ]; then
        rm -rf "$DEST_PATH/enmperor/Save"
        ln -s "/Users/$USER/Documents/Legacy of Empires/Caesar_Saves" "$DEST_PATH/emperor/Save"
    fi
    
    if [[ "$LANGUAGE" == "de-DE" ]]; then
        _helpDefaultWrite "StatusText" "Der erste Kaiser: Aufstieg des Reichs der Mitte läuft"
    else
        _helpDefaultWrite "StatusText" "Emperor is running"
    fi
    
    if [ -d "$DEST_PATH"/emperor/Binks ]; then
        rm -rf "$DEST_PATH"/emperor/Binks
    fi
    
    _helpDefaultWrite "Busy" "1"
    
    open -W "$DEST_PATH/Legacy of Empires.app/Contents/Emperor.app"
    
    _helpDefaultWrite "Busy" "0"
    _helpDefaultWrite "StatusText" ""
}

function play_pharaoh()
{
    install_wrapper
    install_pharaoh
    resolution_pharaoh
    
    if [ ! -L "$DEST_PATH/pharaoh/Save" ]; then
        rm -rf "$DEST_PATH/pharaoh/Save"
        ln -s "/Users/$USER/Documents/Legacy of Empires/Pharaoh_Saves" "$DEST_PATH/pharaoh/Save"
    fi
    
    if [[ "$LANGUAGE" == "de-DE" ]]; then
        _helpDefaultWrite "StatusText" "Pharaoh + Cleopatra läuft"
    else
        _helpDefaultWrite "StatusText" "Pharaoh + CLeopatra is running"
    fi
    
    if [ -f "$DEST_PATH"/pharaoh/BINKS/High/Intro_big.bik ]; then
        rm -rf "$DEST_PATH"/pharaoh/BINKS/High/Intro_big.bik
    fi
    
    _helpDefaultWrite "Busy" "1"
    
    open -W "$DEST_PATH/Legacy of Empires.app/Contents/Pharaoh.app"
    
    _helpDefaultWrite "Busy" "0"
    _helpDefaultWrite "StatusText" ""
}

function play_zeus()
{
    install_wrapper
    install_zeus
    resolution_zeus
    
    if [ ! -L "$DEST_PATH/zeus/Save" ]; then
        rm -rf "$DEST_PATH/zeus/Save"
        ln -s "/Users/$USER/Documents/Legacy of Empires/Zeus_Saves" "$DEST_PATH/zeus/Save"
    fi
    
    if [[ "$LANGUAGE" == "de-DE" ]]; then
        _helpDefaultWrite "StatusText" "Zeus + Poseidon läuft"
    else
        _helpDefaultWrite "StatusText" "Zeus + Poseidon is running"
    fi
    
    if [ -d "$DEST_PATH"/zeus/Binks ]; then
        rm -rf "$DEST_PATH"/zeus/Binks
    fi
    
    _helpDefaultWrite "Busy" "1"
    
    open -W "$DEST_PATH/Legacy of Empires.app/Contents/Zeus.app"
    
    _helpDefaultWrite "Busy" "0"
    _helpDefaultWrite "StatusText" ""
}

function download_german()
{
    if [[ "$LANGUAGE" == "de-DE" ]]; then
        _helpDefaultWrite "StatusText" "Herunterladen/Installation der deutschen Sprachdateien"
    else
        _helpDefaultWrite "StatusText" "Download/Installing of german files"
    fi
}

function install_widescreen()
{
    if [[ "$LANGUAGE" == "de-DE" ]]; then
        _helpDefaultWrite "StatusText" "Herunterladen/Installation vom Widescreen-Patch"
    else
        _helpDefaultWrite "StatusText" "Download/Installing of Widescreenpatch"
    fi
}

function error_gog()
{
    if [[ "$LANGUAGE" == "de-DE" ]]; then
        _helpDefaultWrite "ErrorText" "Fehler! Das ist keine gültige GOG Setup EXE Datei. Versuche es noch einmal."
    else
        _helpDefaultWrite "ErrorText" "Error! This is not the correct setup exe. Please try again."
    fi
    error_sound
    exit 1
}

function error_occured()
{
    if [[ "$LANGUAGE" == "de-DE" ]]; then
        _helpDefaultWrite "ErrorText" "Ein Fehler ist aufgetreten!"
    else
        _helpDefaultWrite "ErrorText" "An Error has occured!"
    fi
    error_sound
    exit 1
}

function _done()
{
    if [[ "$LANGUAGE" == "de-DE" ]]; then
        _helpDefaultWrite "StatusText" ""
    else
        _helpDefaultWrite "StatusText" ""
    fi
}

function error_sound()
{
    afplay ../error.mp3
}

$1
